<?php
/*
Plugin Name: Interstitial Overlays
plugin URI:
Description: Display beautiful interstitial overlays on your site to increase conversions and traffic to other part of your site.
version: 1.0.1
Author: 99 Robots
Author URI: https://99robots.com
License: GPL2
*/

// Exit if accessed directly

if ( !defined( 'ABSPATH' ) ) exit;

if ( !class_exists('NNR_Interstitial_Overlays') ) :

// Set default timezone

date_default_timezone_set(timezone_name_from_abbr(null, (int) get_option('gmt_offset') * 3600 , true));

register_activation_hook( __FILE__, 	array('NNR_Interstitial_Overlays', 'register_activation'));
add_action('admin_menu', 				array('NNR_Interstitial_Overlays', 'menu'));
add_action('init', 						array('NNR_Interstitial_Overlays', 'load_textdoamin'));
add_action('wp_head', 					array('NNR_Interstitial_Overlays', 'display_overlay'));

add_action( 'wp_ajax_nnr_io_set_cookie', 			array('NNR_Interstitial_Overlays', 'set_cookie'));
add_action( 'wp_ajax_nopriv_nnr_io_set_cookie', 	array('NNR_Interstitial_Overlays', 'set_cookie'));

// Include all the helper libraries

require_once('includes/data-manager/autoload.php');
require_once('includes/display-conditions/autoload.php');
require_once('includes/stats-tracker/autoload.php');
require_once('includes/newsletter-integrations/autoload.php');

/**
 * NNR_Interstitial_Overlays class.
 */
class NNR_Interstitial_Overlays {

	/**
	 * prefix
	 *
	 * (default value: 'nnrobots_interstitial_overlays_')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $prefix = 'nnrobots_interstitial_overlays_';

	/**
	 * prefix_dash
	 *
	 * (default value: 'nnr-io-')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $prefix_dash = 'nnr-io-';

	/**
	 * dashboard_page
	 *
	 * (default value: 'nnr-io-dashbaord')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $dashboard_page = 'nnr-io-dashboard';

	/**
	 * text_domain
	 *
	 * (default value: '99robots-interstitial-overlays')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $text_domain = '99robots-interstitial-overlays';

	/**
	 * add_edit_page
	 *
	 * (default value: 'nnr-io-add-edit')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $add_edit_page = 'nnr-io-add-edit';

	/**
	 * email_list_page
	 *
	 * (default value: 'nnr-io-email-list')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $email_list_page = 'nnr-io-email-list';

	/**
	 * updates_page
	 *
	 * (default value: 'nnr-io-updates')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $updates_page = 'nnr-io-updates';

	/**
	 * stats_page
	 *
	 * (default value: 'nnr-io-stats')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $stats_page = 'nnr-io-stats';

	/**
	 * data_manager_table_name
	 *
	 * (default value: 'nnr_interstitial_overlays')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $data_manager_table_name = 'nnr_interstitial_overlays';

	/**
	 * stats_table_name
	 *
	 * (default value: 'nnr_interstitial_overlays_stats')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $stats_table_name = 'nnr_interstitial_overlays_stats';

	/**
	 * newsletter_table_name
	 *
	 * (default value: 'nnr_interstitial_overlays_newsletter')
	 *
	 * @var string
	 * @access public
	 * @static
	 */
	static $newsletter_table_name = 'nnr_interstitial_overlays_newsletter';

	/**
	 * Get the default data
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function default_data() {
		return array(
			'name'					=> __('My Overlay', self::$text_domain),
			'active'				=> 0,
			'start_date'			=> date('Y-m-d H:i:s', mktime(0, 0, 0, date("m"), date("d"), date("Y"))),
			'end_date'				=> date("Y-m-d H:i:s", mktime(0, 0, 0, date("m"), date("d"), date("Y")+20)),
			'display_conditions'	=> array(
				'sitewide'			=> true,
			),
			'args'					=> array(
				'content_type'		=> 'ad',
				'powered_by'		=> true,
				'style'				=> array(
					'background_pattern'	=> 'dots',
				)
			),
		);
	}

	/**
	 * Performs tasks needed upon activation
	 *
	 * @since 1.0.0
	 */
	static function register_activation() {

		// Check if multisite, if so then save as site option

		if ( function_exists('is_multisite') && is_multisite() ) {
			update_site_option(self::$prefix . 'version', NNROBOTS_INTERSTITIAL_OVERLAYS_VERSION_NUM);
		} else {
			update_option(self::$prefix . 'version', NNROBOTS_INTERSTITIAL_OVERLAYS_VERSION_NUM);
		}

		// Create Table upon activation

		$data_manager = new NNR_Data_Manager_v1( self::$data_manager_table_name );
		$data_manager->create_table();

		$stats_tracker = new NNR_Stats_Tracker_v1( self::$stats_table_name );
		$stats_tracker->create_table();

		$newsletter_table = new NNR_Newsletter_Integrations_Submission_v1( self::$newsletter_table_name );
		$newsletter_table->create_table();

	}

	/**
	 * Hooks to 'plugin_action_links_' filter
	 *
	 * @since 1.0.0
	 */
	static function settings_link($links) {
		$settings_link = '<a href="admin.php?page=' . self::$dashboard_page . '">' . __('Overlays', self::$text_domain) . '</a>';
		array_unshift($links, $settings_link);
		return $links;
	}

	/**
	 * Load the text domain
	 *
	 * @since 1.0.0
	 */
	static function load_textdoamin() {
		load_plugin_textdomain(self::$text_domain, false, basename( dirname( __FILE__ ) ) . '/languages' );
	}

	/**
	 * Display the Overlay
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function display_overlay( $data_id = null ) {

		global $post;

		// Display a specific overlay

		if ( !empty($data_id) ) {

			$data_manager = new NNR_Data_Manager_v1( self::$data_manager_table_name );
			$data = $data_manager->get_data_from_id($data_id);

			echo self::display_content_type($data);

		}

		// Loop thorugh all overlays and find the one that is should display

		else {

			$now = date('Y-m-d h:m:s');

			$data_manager = new NNR_Data_Manager_v1( self::$data_manager_table_name );
			$data = $data_manager->get_data_from_date_range($now, $now);
			$display_manager = new NNR_Display_Conditions_Display_v1();

			$sitewide = array();
			$taxonomies = array();
			$post_ids = array();

			foreach ( $data as $item ) {

				// Sitewide

				if ( $item['display_conditions']['sitewide'] ) {
					$sitewide[] = $item;
					continue;
				}

				if ( $item['display_conditions']['frontpage'] ) {
					$post_ids[] = $item;
					continue;
				}

				// Posts

				if ( $item['display_conditions']['post_types'][$post->post_type]['type'] != 'none' &&
					 $item['display_conditions']['post_types'][$post->post_type]['type'] != 'all' &&
					 $item['display_conditions']['post_types'][$post->post_type]['type'] != 'specific' ) {

					$post_ids[] = $item;
					continue;
				} else {
					$taxonomies[] = $item;
					continue;
				}

			}

			$data = array_merge($post_ids, $taxonomies, $sitewide);

			foreach ( $data as $item ) {
				if ( $display_manager->check_conditions($item['display_conditions']) && self::cookie_not_set($item) ) {
					echo self::display_content_type($item);
					break;
				}
			}

		}

	}

	/**
	 * Create the code for the content types
	 *
	 * @access public
	 * @param mixed $item
	 * @return void
	 */
	static function display_content_type( $item ) {

		// Powered By

		$powered_by = '';

		if ( isset($item['args']['powered_by']) && $item['args']['powered_by'] ) {
			$powered_by = '<a class="' . self::$prefix_dash . 'powered-by" href="https://99robots.com/?post_type=download&p=10628" target="_blank" >' . __('Powered By Interstitial Overlays', self::$text_domain) . '</a>';
		}

		// Wrap Background Style

		$background = '';

		// Backgound Image

		if ( isset($item['args']['style']['background_type']) && $item['args']['style']['background_type'] == 'image' ) {

			$background_image_attributes = wp_get_attachment_image_src($item['args']['style']["background_image"], 'full'); // returns an array

			if( $background_image_attributes ) {
				$background = 'style="background-image: url(' . $background_image_attributes[0] . '); background-size: ' . $item['args']['style']['background_size'] . '; background-repeat: ' . $item['args']['style']['background_repeat'] . '; background-position: ' . $item['args']['style']['background_position'] . ';';
			}

		}

		else if ( isset($item['args']['style']['background_type']) && $item['args']['style']['background_type'] == 'pattern' ) {

			// Dots

			if ($item['args']['style']['background_pattern'] == 'dots') {
				$background = 'style="background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAAAXNSR0IArs4c6QAAAB1JREFUGBljYGBg4AFiQoAYNYTMGJWHhgAxgckDACfCADFzWBHCAAAAAElFTkSuQmCC);';
			}

			// Diagonal right

			else if ($item['args']['style']['background_pattern'] == 'diagonal_right') {
				$background = 'style="background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAAAXNSR0IArs4c6QAAABVJREFUGBljYCAO8BCjbFQRA/WCAABg/gB584t6RwAAAABJRU5ErkJggg==);';
			}

			// Diagonal left

			else if ($item['args']['style']['background_pattern'] == 'diagonal_left') {
				$background = 'style="background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAAAXNSR0IArs4c6QAAABhJREFUGBljYGBg4AFiosGoYuSgomFoAABg/gB5Q6KWYgAAAABJRU5ErkJggg==);';
			}

			// Grid

			else if ($item['args']['style']['background_pattern'] == 'grid') {
				$background = 'style="background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAAAXNSR0IArs4c6QAAAB1JREFUGBljZGBg4AFiogBRCpmIMgqoaFQh3pACADLgACqd10PPAAAAAElFTkSuQmCC);';
			}

			// Zig Zag

			else if ($item['args']['style']['background_pattern'] == 'zig_zag') {
				$background = 'style="background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAICAYAAADA+m62AAAAAXNSR0IArs4c6QAAADFJREFUGBljYMAEPJhCmCIwRTAaUwVQBF0SnQ/WhFUQXTMuRTBrCcnD1OG1EUURaRwAYKwAecAGMfQAAAAASUVORK5CYII=);';
			}

			// Diamonds

			else if ($item['args']['style']['background_pattern'] == 'diamonds') {
				$background = 'style="background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAAAXNSR0IArs4c6QAAAEVJREFUGBljYMAEPEAhEMYLkBUgs1E0YZPAEMMQQDICLgdnIEmiM3mY0EUI8fGZiiGHIQA0HZsY2FJkCWQ2VheBFGAoAgCw1ADb2LiCOQAAAABJRU5ErkJggg==);';
			}

			// Hearts

			else if ($item['args']['style']['background_pattern'] == 'hearts') {
				$background = 'style="background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAAAXNSR0IArs4c6QAAADlJREFUGBljYKA14EGzAM5nRJKACyKJwZhfmGAsQjSyiSC12Ez9ApJAV4iuGKwIJIgLYDMZl1oyxQFOKAMqAtJ4RAAAAABJRU5ErkJggg==);';
			}

			// Circles

			else if ($item['args']['style']['background_pattern'] == 'circles') {
				$background = 'style="background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAAAXNSR0IArs4c6QAAADNJREFUGBljYKAH4AFaAsJ4AbICZDaKJmwScDEmFKUkcOAmAPUgs7EaAVJAUBFWnSQJAgBkhAB7H4o0xwAAAABJRU5ErkJggg==);';
			}

			// X Out

			else if ($item['args']['style']['background_pattern'] == 'x_out') {
				$background = 'style="background: url(data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAoAAAAKCAYAAACNMs+9AAAAAXNSR0IArs4c6QAAADRJREFUGBljYKAl4MFiODYxsDJkCWQ2FjMYGEAKMBQxYVVKhCCySchsFK3YJLCJoWginwMAZIQAe7Ze2SMAAAAASUVORK5CYII=);';
			}

			else {
				$background = 'style="';
			}
		}

		if ( isset($item['args']['style']['background_color']) && $item['args']['style']['background_color'] != '' ) {
			$background .= 'background-color:' . $item['args']['style']['background_color'] . ';';
		}

		$background .= '"';

		// Header Style

		$header_style = '';

		if ( isset($item['args']['style']['header_text_color']) && $item['args']['style']['header_text_color'] != '' ) {
			$header_style .= 'color:' . $item['args']['style']['header_text_color'] . ';';
		}

		if ( isset($item['args']['style']['header_background_color']) && $item['args']['style']['header_background_color'] != '' ) {
			$header_style .= 'background-color:' . $item['args']['style']['header_background_color'] . ';';
		}

		if ( isset($item['args']['style']['header_border_color']) && $item['args']['style']['header_border_color'] != '' ) {
			$header_style .= 'border-color:' . $item['args']['style']['header_border_color'] . ';';
		}

		// Header Text

		if ( isset($item['args']['use_close_button']) && $item['args']['use_close_button'] ) {
			$header_text = '<i class="fa fa-times"></i>';
		} else {
			$header_text = isset($item['args']['continue_button_text']) ? esc_attr($item['args']['continue_button_text']) : '';
		}

		if ( $item['args']['content_type'] == 'age-verifier' || $item['args']['content_type'] == 'acceptance' ) {
			$header = '';
		} else {
			$header = '<div style="' . $header_style . '" class="' . self::$prefix_dash . 'header">' . $header_text . '</div>';
		}

		$code = '<div class="' . self::$prefix_dash . 'wrap ' . self::$prefix_dash . 'wrap-' . $item['args']['content_type'] . '" ' . $background . '>' . $header .
			'<div class="' . self::$prefix_dash . 'container ' . self::$prefix_dash . 'container-size-' . $item['args']['content_size'] . '">
				<div class="' . self::$prefix_dash . 'content">' . apply_filters('the_content', html_entity_decode($item['args']['content'])) . '</div>
				<div class="' . self::$prefix_dash . 'content-type">';

		switch ( $item['args']['content_type'] ) {

			case 'ad':

				$ad_image_src = '';

				if ( $item['args']['ad_type'] == 'image' ) {
					$ad_image_attributes = wp_get_attachment_image_src($item['args']['ad_image'], 'full');

					if( $ad_image_attributes ) {
						$ad_image_src = $ad_image_attributes[0];
					} else {
						$ad_image_src = '';
					}
				} else {
					$ad_image_src = $item['args']['ad_image_link'];
				}

				$code .= '<div class="' . self::$prefix_dash . $item['args']['content_type'] . '-wrap">
					<a class="' . self::$prefix_dash . 'ad-link" href="' . esc_url($item['args']['ad_destination_link']) . '" target="' . $item['args']['ad_destination_target'] . '" ' . (isset($item['args']['ad_destination_rel']) && $item['args']['ad_destination_rel'] ? 'rel="nofollow"' : '') . '>
						<img src="' . $ad_image_src . '"/>
					</a>
				</div></div>';

			break;

			case 'third-ad':

				$code .= '<div class="' . self::$prefix_dash . $item['args']['content_type'] . '-wrap">' . html_entity_decode($item['args']["third_ad_code"]) . '</div></div>';

			break;

			case 'announcement':

				$code .= '</div>';

			break;

			case 'button':

				$code .= '<div class="' . self::$prefix_dash . $item['args']['content_type'] . '-wrap">
					<a class="' . self::$prefix_dash . 'button-link" href="' . esc_url($item['args']['button_link']) . '" target="' . $item['args']['button_target'] . '" ' . (isset($item['args']['button_rel']) && $item['args']['button_rel'] ? 'rel="nofollow"' : '') . '><button style="background-color:' . $item['args']['button_background_color'] . ';color:' . $item['args']['button_text_color'] . ';">' . $item['args']['button_text'] . '</button></a>
				</div></div>';

			break;

			case 'age-verifier':

				$code .= '<div class="' . self::$prefix_dash . $item['args']['content_type'] . '-wrap">
				    <div class="' . self::$prefix_dash . 'age-verifier-fields">
				   		<div class="' . self::$prefix_dash . 'age-verifier-fields-group">
				    		<input type="text" class="' . self::$prefix_dash . 'age-verifier-fields-input ' . self::$prefix_dash . 'age-verifier-month" placeholder="MM" />
				    	</div>
				    	<div class="' . self::$prefix_dash . 'age-verifier-fields-group">
				    		<input type="text" class="' . self::$prefix_dash . 'age-verifier-fields-input ' . self::$prefix_dash . 'age-verifier-day" placeholder="DD" />
				    	</div>
				    	<div class="' . self::$prefix_dash . 'age-verifier-fields-group">
							<input type="text" class="' . self::$prefix_dash . 'age-verifier-fields-input ' . self::$prefix_dash . 'age-verifier-year" placeholder="YYYY" />
						</div>
				    </div>
					<button class="' . self::$prefix_dash . 'age-verifier-button" style="background-color:' . $item['args']['age_verifier_button_background_color'] . ';color:' . $item['args']['age_verifier_button_text_color'] . ';">' . $item['args']['age_verifier_button_text'] . '</button>
				</div></div>';

				wp_enqueue_script('moment', NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/js/moment.js');

			break;

			case 'acceptance':

				$code .= '<div class="' . self::$prefix_dash . $item['args']['content_type'] . '-wrap">';

					if ( isset($item['args']['require_checkbox']) && $item['args']['require_checkbox'] ) {
						$code .= '<input type="checkbox" class="' . self::$prefix_dash . 'acceptance-checkbox" />
						<span class="' . self::$prefix_dash . 'acceptance-text">' . $item['args']['confirm_text'] . '</span>';
					}

					$code .= '<button class="' . self::$prefix_dash . 'acceptance-button" style="background-color:' . $item['args']['acceptance_button_background_color'] . ';color:' . $item['args']['acceptance_button_text_color'] . ';">' . $item['args']['acceptance_button_text'] . '</button>
				</div></div>';

			break;

			case 'optin':

				$newsletter_form = new NNR_Newsletter_Integrations_Form_v1( self::$prefix_dash, self::$text_domain, self::$data_manager_table_name, self::$newsletter_table_name, self::$stats_table_name);

				$code .= '<div class="' . self::$prefix_dash . $item['args']['content_type'] . '-wrap">' . $newsletter_form->display_form($item['id'], $item['args']['newsletter'], $item['args']['newsletter']) . '</div></div>';

			break;

			default:

				$code .= '</div>';

			break;
		}

			$code .= $powered_by .
			'</div>
		</div>';

		// Styles

		wp_enqueue_style(self::$prefix_dash . 'css', plugins_url('css/interstitial-overlays.css', __FILE__));
		wp_enqueue_style(self::$prefix_dash . 'fontawesome_css', plugins_url('css/font-awesome.min.css', __FILE__));

		// Scripts

		wp_enqueue_script(self::$prefix_dash . 'js', plugins_url('js/interstitial-overlays.js', __FILE__));
		wp_localize_script(self::$prefix_dash . 'js', 'nnr_io_data', array(
			'prefix'					=> self::$prefix_dash,
			'ajaxurl'					=> admin_url('admin-ajax.php'),
			'stats_table_name'			=> self::$stats_table_name,
			'data_id'					=> $item['id'],
			'close_button'				=> isset($item['args']['use_close_button']) && $item['args']['use_close_button'] ? true : false,
			'wait_time'					=> isset($item['args']['wait_time']) && $item['args']['wait_time'] != '' ? $item['args']['wait_time'] : '5',
			'minimum_age'				=> isset($item['args']['minimum_age']) && $item['args']['minimum_age'] != '' ? $item['args']['minimum_age'] : '18',
			'age_verifier_error_text'	=> isset($item['args']['age_verifier_error_text']) && $item['args']['age_verifier_error_text'] != '' ? $item['args']['age_verifier_error_text'] : __('You are not old enough to enter the site.', self::$prefix_dash),
		));

		return $code;

	}

	/**
	 * This function is called via AJAX to set the cookie of an overlay to not show it again
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function set_cookie(){

		if ( !isset($_POST['data_id']) ) {
			echo __('No data id found so cookie could not be set.');
			die();
		}

		if ( is_admin() ) {
			echo __('Cookie was not set because you are in the admin panel.');
			die();
		}

		setcookie(self::$prefix . $_POST['data_id'] . '_dont_show_again', true, 0, '/');

		echo __('Cookie has been set!');
		die();

	}

	/**
	 * Check if this overlay has a cookie set for it and if we should display it
	 *
	 * @access public
	 * @static
	 * @param mixed $item
	 * @return void
	 */
	static function cookie_not_set( $item ){

		// Dont show again is set to false so always show

		if ( isset($item['args']['dont_show_again']) && !$item['args']['dont_show_again'] ) {
			return true;
		}

		// Dont show again is set to true to check the cookie

		else if ( isset($item['args']['dont_show_again']) && $item['args']['dont_show_again'] && isset($_COOKIE[self::$prefix . $item['id'] . '_dont_show_again']) ) {
			return false;
		}

		return true;

	}

	/**
	 * Hooks to 'admin_menu'
	 *
	 * @since 1.0.0
	 */
	static function menu() {

		// Dashboard

		add_menu_page(
			__('Interstitial Overlays', self::$text_domain),			// Page Title
			__('Interstitial Overlays', self::$text_domain), 			// Menu Name
	    	'manage_options', 											// Capabilities
	    	self::$dashboard_page, 										// slug
	    	array('NNR_Interstitial_Overlays', 'dashboard'),			// Callback function
	    	plugin_dir_url(__FILE__) . 'img/logo.png" style="width:20px;padding-top: 6px;'
	    );

	    // Dashboard

	    $dashboard_page_load = add_submenu_page(
	    	self::$dashboard_page, 										// parent slug
	    	__('All Overlays', self::$text_domain), 					// Page title
	    	__('All Overlays', self::$text_domain), 					// Menu name
	    	'manage_options', 											// Capabilities
	    	self::$dashboard_page, 										// slug
	    	array('NNR_Interstitial_Overlays', 'dashboard')				// Callback function
	    );
	    add_action("admin_print_scripts-$dashboard_page_load", array('NNR_Interstitial_Overlays', 'dashboard_scripts'));

	    // Add / Edit

	    $add_edit_page_load = add_submenu_page(
	    	self::$dashboard_page, 										// parent slug
	    	__('Add New', self::$text_domain), 							// Page title
	    	__('Add New', self::$text_domain), 							// Menu name
	    	'manage_options', 											// Capabilities
	    	self::$add_edit_page, 										// slug
	    	array('NNR_Interstitial_Overlays', 'add_edit')				// Callback function
	    );
	    add_action("admin_print_scripts-$add_edit_page_load", array('NNR_Interstitial_Overlays', 'add_edit_scripts'));

	    // Stats

	    $stats_page_load = add_submenu_page(
	    	self::$dashboard_page, 										// parent slug
	    	__('Stats', self::$text_domain), 							// Page title
	    	__('Stats', self::$text_domain), 							// Menu name
	    	'manage_options', 											// Capabilities
	    	self::$stats_page, 											// slug
	    	array('NNR_Interstitial_Overlays', 'stats')					// Callback function
	    );
	    add_action("admin_print_scripts-$stats_page_load", array('NNR_Interstitial_Overlays', 'stats_scripts'));

	    // Email List

	    $email_list_load = add_submenu_page(
	    	self::$dashboard_page, 										// parent slug
	    	__('Email List', self::$text_domain),  						// Page title
	    	__('Email List', self::$text_domain),  						// Menu name
	    	'manage_options', 											// Capabilities
	    	self::$email_list_page, 									// slug
	    	array('NNR_Interstitial_Overlays', 'email_list')			// Callback function
	    );
	    add_action("admin_print_scripts-$email_list_load", array('NNR_Interstitial_Overlays', 'email_list_scripts'));

	    // Another sub menu

	    $updates_page_load = add_submenu_page(
	    	self::$dashboard_page, 										// parent slug
	    	__('Updates', self::$text_domain),  						// Page title
	    	__('Updates', self::$text_domain),  						// Menu name
	    	'manage_options', 											// Capabilities
	    	self::$updates_page, 										// slug
	    	array('NNR_Interstitial_Overlays', 'updates')				// Callback function
	    );
	    add_action("admin_print_scripts-$updates_page_load", array('NNR_Interstitial_Overlays', 'updates_scripts'));
	}

	/**
	 * Hooks to "admin_print_scripts-$page"
	 *
	 * @since 1.0.0
	 *
	 * @param 	N/A
	 * @return 	N/A
	 */
	static function dashboard_scripts() {

		// Styles

		wp_enqueue_style(self::$prefix . 'settings_css', 					NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/settings.css');
		wp_enqueue_style(self::$prefix . 'fontawesome_css', 				NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/font-awesome.min.css');
	    wp_enqueue_style(self::$prefix . 'bootstrap_css', 					NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/nnr-bootstrap.min.css');
	    wp_enqueue_style('wp-color-picker');

		// Scripts

		wp_enqueue_script(self::$prefix . 'bootstrap_js', 					NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/js/bootstrap.min.js', array('jquery'));
		wp_enqueue_script(self::$prefix . 'admin_js', 						NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/js/dashboard.js', array('wp-color-picker', 'jquery'), false, true );
	}

	/**
	 * Hooks to "admin_print_scripts-$page"
	 *
	 * @since 1.0.0
	 *
	 * @param 	N/A
	 * @return 	N/A
	 */
	static function add_edit_scripts() {

	    // Styles

	    wp_enqueue_style(self::$prefix . 'settings_css', 					NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/settings.css');
		wp_enqueue_style(self::$prefix . 'bootstrap_css', 					NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/nnr-bootstrap.min.css');
		wp_enqueue_style(self::$prefix . 'fontawesome_css', 				NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/font-awesome.min.css');
		wp_enqueue_style(self::$prefix . 'bootstrap_vertical_tabs_css',		NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/bootstrap-helpers/bootstrap.vertical-tabs.min.css');
		wp_enqueue_style(self::$prefix . 'bootstrap_slider_css', 			NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/bootstrap-helpers/bootstrap-slider.css');
		wp_enqueue_style(self::$prefix . 'bootstrap_iconpicker_css', 		NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/bootstrap-helpers/bootstrap-iconpicker.min.css');
		wp_enqueue_style(self::$prefix . 'bootstrap_formhelpers_css', 		NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/bootstrap-helpers/bootstrap-formhelpers.min.css');

	    // Scripts

		wp_enqueue_script(self::$prefix . 'bootstrap_js', 					NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/js/bootstrap.min.js', array('jquery'));
		wp_enqueue_script(self::$prefix . 'bootstrap_slider_js', 			NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/js/bootstrap-helpers/bootstrap-slider.js', array('jquery'));
		wp_enqueue_script(self::$prefix . 'bootstrap_iconpicker_iconset_js',NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/js/bootstrap-helpers/iconset-fontawesome-4.2.0.min.js', array('jquery', self::$prefix . 'bootstrap_js'));
		wp_enqueue_script(self::$prefix . 'bootstrap_iconpicker_js', 		NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/js/bootstrap-helpers/bootstrap-iconpicker.js', array('jquery', self::$prefix . 'bootstrap_js', self::$prefix . 'bootstrap_iconpicker_iconset_js'));
		wp_enqueue_script(self::$prefix . 'bootstrap_formhelpers_js', 		NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/js/bootstrap-helpers/bootstrap-formhelpers.js', array('jquery'));

	    wp_enqueue_style('wp-color-picker');
	    wp_enqueue_script(self::$prefix . 'admin_add_edit_js',				NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/js/add-edit.js', array('wp-color-picker', 'jquery', 'jquery-ui-tabs', 'jquery-ui-datepicker'), false, true
	    );
	    wp_localize_script(self::$prefix . 'admin_add_edit_js', 'nnr_io_add_edit_data', array(
		   'prefix'	=> self::$prefix_dash,
	    ));
	}

	/**
	 * Hooks to "admin_print_scripts-$page"
	 *
	 * @since 1.0.0
	 *
	 * @param 	N/A
	 * @return 	N/A
	 */
	static function stats_scripts() {

		// Styles

		wp_enqueue_style(self::$prefix . 'settings_css', 					NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/settings.css');
	    wp_enqueue_style(self::$prefix . 'fontawesome_css', 				NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/font-awesome.min.css');
	    wp_enqueue_style(self::$prefix . 'bootstrap_css', 					NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/nnr-bootstrap.min.css');

	    // Scripts

		wp_enqueue_script(self::$prefix . 'bootstrap_js', 					NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/js/bootstrap.min.js', array('jquery'));

	}

	/**
	 * Hooks to "admin_print_scripts-$page"
	 *
	 * @since 1.0.0
	 *
	 * @param 	N/A
	 * @return 	N/A
	 */
	static function email_list_scripts() {

		// Styles

		wp_enqueue_style(self::$prefix . 'settings_css', 					NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/settings.css');
		wp_enqueue_style(self::$prefix . 'bootstrap_css', 					NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/nnr-bootstrap.min.css');
	    wp_enqueue_style(self::$prefix . 'fontawesome_css', 				NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/font-awesome.min.css');

	    // Scripts

		wp_enqueue_script(self::$prefix . 'bootstrap_js', 					NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/js/bootstrap.min.js', array('jquery'));
	}

	/**
	 * Hooks to "admin_print_scripts-$page"
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function updates_scripts(){

		wp_enqueue_style(self::$prefix . 'settings_css', 					NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/settings.css');
	    wp_enqueue_style(self::$prefix . 'fontawesome_css', 				NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/font-awesome.min.css');
	    wp_enqueue_style(self::$prefix . 'bootstrap_css', 					NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/css/nnr-bootstrap.min.css');

	    wp_enqueue_script(self::$prefix . 'updates_js', 					NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL . '/js/updates.js', array('jquery'));
	    wp_localize_script(self::$prefix . 'updates_js', 'nnr_io_updates_data', array(
			'prefix_dash' 		=> self::$prefix_dash,
			'activate_text'		=> __('Activate', self::$text_domain),
			'deactivate_text'	=> __('Deactivate', self::$text_domain),
			'no_license_key'		=> __('Please add or activate your License Key to get automatic updates.  Without a valid license key you will not receive regular updates. You can find your license key', self::$text_domain) . ' <a href="https://99robots.com/dashboard" target="_blank">here</a>',
			'expired'				=> __('EXPIRED: This license key has expired.  Please renew your license key', self::$text_domain) . ' <a href="https://99robots.com/dashboard" target="_blank">here</a>',
			'activation_error'		=> __('INVALID LICENSE KEY: The license key is not valid please try again. You can find your license key', self::$text_domain) . ' <a href="https://99robots.com/dashboard" target="_blank">here</a>.',
			'activation_valid'		=> __('SUCCESS: Your license key is valid.', self::$text_domain),
			'deactivation_valid'	=> __('SUCCESS: This site has been deactivated.', self::$text_domain),
			'deactivation_error'	=> __('DEACTIVATION FAILED: This site could not be deactivated.', self::$text_domain),
		));

	}

	/**
	 * Display the Dashboard page
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function dashboard() {

		include_once(NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_DIR . 'controllers/dashboard.php');

	}

	/**
	 * Display the Add New page
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function add_edit() {

		include_once(NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_DIR . 'controllers/add-edit.php');

	}

	/**
	 * Display the Stats page
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function stats() {

		include_once(NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_DIR . 'controllers/stats.php');

	}

	/**
	 * Display the Email List page
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function email_list() {

		include_once(NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_DIR . 'controllers/email-list.php');

	}

	/**
	 * Display the Updates page
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function updates() {

		include_once(NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_DIR . 'controllers/updates.php');

	}

	/**
	 * Perform all the actions with a license
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function license_action() {

		if ( !isset($_POST['license_key']) ) {
			echo '';
			die();
		}

		// data to send in our API request

		$api_params = array(
			'edd_action'	=> $_POST['license_action'],
			'license' 		=> $_POST['license_key'],
			'item_name' 	=> urlencode( NNROBOTS_INTERSTITIAL_OVERLAYS_ITEM_NAME ), // the name of our product in EDD
			'url'       	=> is_multisite() ? network_home_url() : home_url()
		);

		// Call the custom API.

		$response = wp_remote_post( NNROBOTS_INTERSTITIAL_OVERLAYS_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		// make sure the response came back okay

		if ( is_wp_error( $response ) ) {
			echo $response;
			die();
		}

		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		if ( $_POST['license_action'] == 'activate_license' ) {
			self::update_license_key($_POST['license_key']);
		} else if ( $_POST['license_action'] == 'deactivate_license' && $license_data->license == 'deactivated') {
			self::update_license_key('');
		}

		// decode the license data

		echo $license_data->license;

		die();

	}

	/**
	 * Check if license key is active
	 *
	 * @access public
	 * @static
	 * @param mixed $license_key
	 * @return void
	 */
	static function is_license_active( $license_key ) {

		if ( !isset( $license_key ) ) {
			return false;
		}

		// data to send in our API request

		$api_params = array(
			'edd_action'	=> 'check_license',
			'license' 		=> $license_key,
			'item_name' 	=> urlencode( NNROBOTS_INTERSTITIAL_OVERLAYS_ITEM_NAME ), // the name of our product in EDD
			'url'       	=> is_multisite() ? network_home_url() : home_url()
		);

		// Call the custom API.

		$response = wp_remote_post( NNROBOTS_INTERSTITIAL_OVERLAYS_STORE_URL, array( 'timeout' => 15, 'sslverify' => false, 'body' => $api_params ) );

		// make sure the response came back okay

		if ( is_wp_error( $response ) ) {
			return false;
		}

		$license_data = json_decode( wp_remote_retrieve_body( $response ) );

		// decode the license data

		if ( isset($license_data->license) && $license_data->license == 'valid' ) {
			return true;
		} else {
			return false;
		}

	}

	/**
	 * Get the license key
	 *
	 * @access public
	 * @static
	 * @return void
	 */
	static function get_license_key() {

		// Get the license key from based on WordPress install

		if ( function_exists('is_multisite') && is_multisite() ) {
			$license_key = get_site_option(self::$prefix . 'license_key');
		} else {
			$license_key = get_option(self::$prefix . 'license_key');
		}

		// Check if license key is set

		if ( $license_key === false ) {
			$license_key = '';
		}

		return $license_key;
	}

	/**
	 * Update the license key
	 *
	 * @access public
	 * @static
	 * @param mixed $license_key
	 * @return void
	 */
	static function update_license_key($license_key) {

		if ( isset( $license_key ) ) {

			// Get the license key from based on WordPress install

			if ( function_exists('is_multisite') && is_multisite() ) {

				update_site_option(self::$prefix . 'license_key', $license_key);

			} else {

				update_option(self::$prefix . 'license_key', $license_key);

			}
		}
	}

}

// Item Name

if ( !defined('NNROBOTS_INTERSTITIAL_OVERLAYS_ITEM_NAME') ) {
	define('NNROBOTS_INTERSTITIAL_OVERLAYS_ITEM_NAME', 'Interstitial Overlays');
}

// Store URL

if ( !defined('NNROBOTS_INTERSTITIAL_OVERLAYS_STORE_URL') ) {
	define('NNROBOTS_INTERSTITIAL_OVERLAYS_STORE_URL', 'https://99robots.com');
}

// Minimum PHP version

if ( !defined('NNROBOTS_INTERSTITIAL_OVERLAYS_MIN_PHP_VERSION') ) {
	define('NNROBOTS_INTERSTITIAL_OVERLAYS_MIN_PHP_VERSION', '5.4.0');
}

// Plugin Name

if ( !defined('NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_NAME') ) {
	define('NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_NAME', trim(dirname(plugin_basename(__FILE__)), '/'));
}

// Plugin directory

if ( !defined('NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_DIR') ) {
	define('NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_DIR', plugin_dir_path(__FILE__) );
}

// Plugin url

if ( !defined('NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL') ) {
	define('NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_URL', plugins_url() . '/' . NNROBOTS_INTERSTITIAL_OVERLAYS_PLUGIN_NAME);
}

// Plugin verison

if ( !defined('NNROBOTS_INTERSTITIAL_OVERLAYS_VERSION_NUM') ) {
	define('NNROBOTS_INTERSTITIAL_OVERLAYS_VERSION_NUM', '1.0.1');
}

if ( !class_exists('EDD_SL_Plugin_Updater') ) {
	include_once( dirname( __FILE__ ) . '/includes/edd-software-licensing-updates/EDD_SL_Plugin_Updater.php' );
}

add_action('wp_ajax_nnr_io_license_action', array('NNR_Interstitial_Overlays', 'license_action'));

$edd_updater = new EDD_SL_Plugin_Updater( NNROBOTS_INTERSTITIAL_OVERLAYS_STORE_URL , __FILE__, array(
		'version' 	=> NNROBOTS_INTERSTITIAL_OVERLAYS_VERSION_NUM,
		'license' 	=> NNR_Interstitial_Overlays::get_license_key(),
		'item_name' => NNROBOTS_INTERSTITIAL_OVERLAYS_ITEM_NAME,
		'author' 	=> '99 Robots',
	)
);

endif;